import os
import time
from http.server import HTTPServer, BaseHTTPRequestHandler
import socketserver
import errno

# 2025-03-14 23:46:39 
# https://grok.com/share/bGVnYWN5_1bb998cb-cb3d-4b66-b145-4d9f60504eae
# Verzeichnis mit MP3-Dateien oder M3U-Datei
# MUSIC_DIR = "./music"
# M3U_FILE = "playlist.m3u"  # Optional

# Kopiere diese Datei in denselben Ordner, in dem sich die MP3 oder M3U befinden
MUSIC_DIR = os.path.dirname(os.path.abspath(__file__))
M3U_FILE = os.path.join(MUSIC_DIR, "playlist.m3u")

class RadioStreamHandler(BaseHTTPRequestHandler):
    def do_GET(self):
        if self.path == "/stream":
            self.send_response(200)
            self.send_header("Content-type", "audio/mpeg")
            self.send_header("Transfer-Encoding", "chunked")
            self.send_header("Cache-Control", "no-cache")
            self.send_header("Connection", "keep-alive")
            self.send_header("icy-name", "Mein LAN-Radio")
            self.end_headers()

            playlist = self.server.get_playlist()
            if not playlist:
                self.wfile.write(b"0\r\n\r\n")
                return

            while True:
                for mp3_file in playlist:
                    try:
                        with open(mp3_file, "rb") as f:
                            audio_data = f.read()
                            chunk_size = len(audio_data)
                            self.wfile.write(f"{chunk_size:x}\r\n".encode())
                            self.wfile.write(audio_data)
                            self.wfile.write(b"\r\n")
                            self.wfile.flush()
                            print(f"Spiele: {mp3_file}")
                    except IOError as e:
                        if e.errno == errno.EPIPE:  # Broken Pipe
                            print(f"Client getrennt bei {mp3_file}, fahre fort...")
                            return  # Beendet diesen Request, aber Server läuft weiter
                        else:
                            print(f"Fehler bei {mp3_file}: {e}")
                            return
                    except Exception as e:
                        print(f"Unbekannter Fehler bei {mp3_file}: {e}")
                        return
                time.sleep(0.1)  # Pause zwischen Songs

        else:
            self.send_error(404, "Nur /stream verfügbar")

class RadioServer(HTTPServer):
    def __init__(self, server_address, handler_class):
        super().__init__(server_address, handler_class)
        self.playlist = self.load_playlist()

    def load_playlist(self):
        if os.path.exists(M3U_FILE):
            with open(M3U_FILE, "r") as f:
                return [line.strip() for line in f if line.endswith(".mp3")]
        else:
            return [os.path.join(MUSIC_DIR, f) for f in os.listdir(MUSIC_DIR) if f.endswith(".mp3")]

    def get_playlist(self):
        return self.playlist

def run():
    server_address = ("0.0.0.0", 8000)  # Bindet an alle Interfaces im LAN
    httpd = RadioServer(server_address, RadioStreamHandler)
    local_ip = get_local_ip()
    print(f"LAN-Radio läuft auf http://{local_ip}:8000/stream")
    httpd.serve_forever()

def get_local_ip():
    # Lokale IP herausfinden
    import socket
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        s.connect(("8.8.8.8", 80))  # Verbindung zu Google DNS, nur um IP zu bekommen
        ip = s.getsockname()[0]
    except Exception:
        ip = "localhost"
    finally:
        s.close()
    return ip

if __name__ == "__main__":
    run()
